(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     79757,       1558]*)
(*NotebookOutlinePosition[     80494,       1583]*)
(*  CellTagsIndexPosition[     80450,       1579]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["Package Intro", "Subsection",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      MrvLimit\ v1  .0\ \((c)\)\ 2005\ by\ Udo\ Richter\ \ \ \ \ \ \ \ \ \ \ \
*) \)\(\[IndentingNewLine]\)\( (*\ 
      mail : udo_richter \((at)\) 
          gmx . de\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ *) \)\(\
\[IndentingNewLine]\)\( (*\ 
      web : \(\(http\)\(:\)\) // 
        urichter . cjb . net/
          MrvLimit\ \ \ \ \ \ \ \ \ \ \ \ *) \)\(\[IndentingNewLine]\)\( (*\ 
      Released\ under\ the\ GNU\ General\ Public\ \(\(License\)\(.\)\)\ \ \
*) \)\(\[IndentingNewLine]\)\( (*\ "\<$Id: MrvLimit.nb 00 0000-00-00 \
00:00:00Z udo $\>"\ *) \)\)\)], "Input",
  CellLabel->"In[15]:=",
  InitializationCell->True],

Cell["$Id: MrvLimit.nb 97 2005-05-11 00:57:18Z udo $", "Text",
  InitializationCell->True],

Cell[BoxData[
    \(\(BeginPackage["\<MrvLimit`\>"];\)\)], "Input",
  CellLabel->"In[16]:=",
  InitializationCell->True],

Cell[BoxData[{
    \(\(Off[General::spell1];\)\), "\[IndentingNewLine]", 
    \(\(Off[General::spell];\)\), "\[IndentingNewLine]", 
    \(\({power, pwr, log, sin, cos, tan, arcsin, arccos, arctan, gamma, 
        loggamma, polygamma, Output, output, direction, term, Limit, 
        LeadTerm, SignedZero, Debug, Automatic, Direction, ClearCache, 
        MrvLimitGetCacheStats, MrvLimitClearCache, CacheHits, 
        CacheMisses, \[Omega], MrvF, 
        MrvSeriesHead, $MrvSeriesHead, $DebugPrint, IndentPrint, 
        MrvTestZero, $MrvTestZero, 
        MrvTestZeroInterval, $MrvTestZeroInterval, $OutputProcessor, 
        MrvLimitLoadSilent, Zero, FromInternal, 
        ToInternal};\)\), "\[IndentingNewLine]", 
    \(\(On[General::spell1];\)\), "\[IndentingNewLine]", 
    \(\(On[General::spell];\)\)}], "Input",
  CellLabel->"In[17]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(ToInternal::usage = \[IndentingNewLine]"\<ToInternal[f,x] translates \
f[x] to internal package form, making sure that no critical automatic \
transformations will be done on f any more. If ToInternal succeeds, return \
value will be free of ToInternal calls (check using FreeQ). If ToInternal \
fails, an UnsupportedFunction error is issued, and the failing ToInternal \
call will be encapsulated in HoldForm.\>";\)\)], "Input",
  CellLabel->"In[22]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(FromInternal::usage = "\<FromInternal[f,x] translates internal \
representation of f[x] back to general Mathematica representation. See \
ToInternal[f,x] for details.\>";\)\)], "Input",
  CellLabel->"In[23]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(MrvLimit::usage = "\<MrvLimit[f,x->lim,options] calculates the limit \
of f for x approaching lim. If MrvLimit succeeds, the limit value will be \
returned. In  case of two-sided limits, a list of two results may be \
returned, in case that they are different. If MrvLimit fails, an error \
message will be issued, and MrvLimit will return the whole call encapsulated \
in HoldForm. \nPossible Options:\n Direction: One of -1, +1, 0 or Automatic. \
Defaults to Automatic. +1 will calculate limits from below, -1 from above, 0 \
both sided. Automatic does both sided limits for finite limit processes, and \
matching sides for infinite.\n Output: Limit, LeadTerm or SignedZero. Default \
Limit. LeadTerm will return a LeadTerm[] object describing the found \
asymptotic. See LeadTerm help. SignedZero will return Zero or -Zero if the \
limit is 0.\n Debug: positive integer n or Automatic. Default Automatic. \
Enables debugging output down to nested level n in recursive calls. Automatic \
mode will detect recursive calls and inherit predecessor's debug level minus \
1.\>";\)\)], "Input",
  CellLabel->"In[24]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(LeadTerm::usage = \*"\"\<Output->LeadTerm is an option to MrvLimit, \
to return the found asymptotic behavior of the function. To preserve its \
structure, the result will be in internal form, see ToInternal.\\nGeneral \
result form is LeadTerm[c,a], with a of form \
{pwr[\!\(\[Omega]\_1\),\!\(e\_1\)],pwr[\!\(\[Omega]\_2\),\!\(e\_2\)],..}.\\n\!\
\(\[Omega]\_i\) is an asymptotic with \!\(\[Omega]\_i\)\[Rule]0, \!\(\[Omega]\
\_i\)>0 for x\[Rule]\[Infinity]. \!\(e\_i\) is the exponent order of the \
asymptotic, and may be 0. The a list is ordered by dominance, dominating \
asymptotics first. c is the leading term constant.\\nThe general asymptotic \
behavior is c*Apply[Times,a]/.pwr->Power.\>\"";\)\)], "Input",
  CellLabel->"In[25]:=",
  InitializationCell->True],

Cell[BoxData[{
    \(\(MrvLimit::"\<LimitFail\>" = "\<Limit failed at `1` on `2`\>";\)\), "\
\[IndentingNewLine]", 
    \(\(MrvLimit::"\<UnknownSign\>" = "\<Cannot determine sign: `1`\>";\)\), \
"\[IndentingNewLine]", 
    \(\(MrvLimit::"\<SeriesFail\>" = "\<Series failed at `1`\>";\)\), "\
\[IndentingNewLine]", 
    \(\(MrvLimit::"\<SeriesNoTerms\>" = "\<Series failed at `1`: No leading \
term found, possibly undetected Zero function\>";\)\), "\[IndentingNewLine]", \

    \(\(MrvLimit::"\<ZeroFunction\>" = "\<Unexpected Zero function \
detected.\>";\)\), "\[IndentingNewLine]", 
    \(\(MrvLimit::"\<UnknownOutput\>" = "\<Unknown output type \
Output\[Rule]`1`\>";\)\), "\[IndentingNewLine]", 
    \(\(MrvLimit::"\<MrvSetFailed\>" = "\<MrvSet failed at `1`\>";\)\), "\
\[IndentingNewLine]", 
    \(\(MrvLimit::"\<UnsupportedFunction\>" = "\<MrvLimit does not support \
this function: `1`\>";\)\), "\[IndentingNewLine]", 
    \(\(MrvLimit::"\<UnsupportedArgument\>" = "\<MrvLimit does not support \
`1` for `2`, appearing in `3`\>";\)\), "\[IndentingNewLine]", 
    \(\(MrvLimit::"\<RecursiveCall\>" = "\<Recursive Limit call encountered \
on `1`\>";\)\), "\[IndentingNewLine]", 
    \(\(MrvLimit::"\<LostContext\>" = "\<Lost Link to aborted MrvLimit \
context detected. Skipping.\>";\)\), "\[IndentingNewLine]", 
    \(\(MrvLimit::"\<UnknownOption\>" = "\<Unknown Option `1`\>";\)\), "\
\[IndentingNewLine]", 
    \(\(MrvLimitInf::"\<UnknownOption\>" = "\<Unknown Option `1`\>";\)\), "\
\[IndentingNewLine]", 
    \(\(MrvLimit::"\<ConditionCheck\>" = "\<Cannot check important condition \
`1`\>";\)\), "\[IndentingNewLine]", 
    \(\(MrvLimit::"\<Assert\>" = "\<Assertion check failed: `1` on \
`2`\>";\)\)}], "Input",
  CellLabel->"In[26]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(Begin["\<`Private`\>"];\)\)], "Input",
  CellLabel->"In[42]:=",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Global", "Subsection",
  InitializationCell->True],

Cell[BoxData[{
    \(\(Off[General::spell1];\)\), "\[IndentingNewLine]", 
    \(\(Off[General::spell];\)\), "\[IndentingNewLine]", 
    \(\({Res, ResA, ResB};\)\), "\[IndentingNewLine]", 
    \(\(On[General::spell1];\)\), "\[IndentingNewLine]", 
    \(\(On[General::spell];\)\)}], "Input",
  CellLabel->"In[43]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      Greetings\ *) \)\(\[IndentingNewLine]\)\(\(\(If[\(! \
\((MrvLimitLoadSilent === 
              True)\)\)\[IndentingNewLine] (*then*) , \
\[IndentingNewLine]\(Print["\<MrvLimit v1.0 \[Copyright] 2005 by Udo Richter\n\
 mail: udo_richter(at)gmx.de\n web: http://urichter.cjb.net/MrvLimit\n \
Released under the GNU General Public License.\>"];\)\[IndentingNewLine]]\) \
\)\(;\)\( (*\ end\ if\ *) \)\(\[IndentingNewLine]\)\(\ \)\)\)\)], "Input",
  CellLabel->"In[48]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      Translate\ term\ to\ internal\ restricted\ function\ space\ *) \)\(\
\[IndentingNewLine]\)\(\[IndentingNewLine]\)\(\(ToInternal[f_, x_] := 
        f /; FreeQ[f, x];\)\[IndentingNewLine]
    \(ToInternal[x_, x_] := x;\)\[IndentingNewLine]
    \(ToInternal[g_*h_, x_] := 
        ToInternal[g, x]*ToInternal[h, x];\)\[IndentingNewLine]
    \(ToInternal[g_ + h_, x_] := 
        ToInternal[g, x] + ToInternal[h, x];\)\[IndentingNewLine]
    \(ToInternal[Power[g_, c_], x_] := 
        power[ToInternal[g, x], ToInternal[c, x]] /; 
          FreeQ[c, x];\)\[IndentingNewLine]
    \(ToInternal[Power[E, g_], x_] := 
        power[E, ToInternal[g, x]];\)\[IndentingNewLine]
    \(ToInternal[Power[f_, g_], x_] := 
        power[E, 
          log[ToInternal[f, x]]*ToInternal[g, x]];\)\[IndentingNewLine]
    \(ToInternal[Log[g_], x_] := 
        log[ToInternal[g, x]];\)\[IndentingNewLine]\[IndentingNewLine]
    \(ToInternal[Sin[f_], x_] := sin[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[Cos[f_], x_] := cos[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[Tan[f_], x_] := tan[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[Sec[f_], x_] := 
        1/cos[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[Csc[f_], x_] := 
        1/sin[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[Cot[f_], x_] := 
        1/tan[ToInternal[f, x]];\)\[IndentingNewLine]\[IndentingNewLine]
    \(ToInternal[ArcSin[f_], x_] := 
        arcsin[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[ArcCos[f_], x_] := 
        arccos[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[ArcTan[f_], x_] := 
        arctan[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[ArcSec[f_], x_] := 
        arccos[1/ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[ArcCsc[f_], x_] := 
        arcsin[1/ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[ArcCot[f_], x_] := 
        arctan[1/ToInternal[f, x]];\)\[IndentingNewLine]\[IndentingNewLine]
    \(ToInternal[Gamma[f_], x_] := 
        gamma[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[LogGamma[f_], x_] := 
        loggamma[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[PolyGamma[n_, f_], x_] := 
        polygamma[n, 
          ToInternal[f, x]];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Pass - through\ if\ already\ in\ internal\ \(\(form\)\(:\)\)\ \
*) \[IndentingNewLine]
    \(ToInternal[power[f_, g_], x_] := 
        power[ToInternal[f, x], ToInternal[g, x]];\)\[IndentingNewLine]
    \(ToInternal[log[f_], x_] := log[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[sin[f_], x_] := sin[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[cos[f_], x_] := cos[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[tan[f_], x_] := tan[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[arcsin[f_], x_] := 
        arcsin[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[arccos[f_], x_] := 
        arccos[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[arctan[f_], x_] := 
        arctan[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[gamma[f_], x_] := 
        gamma[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[loggamma[f_], x_] := 
        loggamma[ToInternal[f, x]];\)\[IndentingNewLine]
    \(ToInternal[polygamma[n_, f_], x_] := 
        polygamma[n, 
          ToInternal[f, x]];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Catch\ all\ \(\(remaining\)\(:\)\)\ *) \[IndentingNewLine]
    \(ToInternal[f_, 
          x_] := \((\[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedFunction\>", 
            f]; \[IndentingNewLine]HoldForm[
            ToInternal[f, 
              x]]\[IndentingNewLine])\);\)\[IndentingNewLine]\
\[IndentingNewLine] (*\ 
      Remove\ all\ internal\ \(\(stuff\)\(:\)\)\ *) \[IndentingNewLine]
    \(FromInternal[f_, x_] := 
        f /. {log \[Rule] Log, power \[Rule] Power, sin \[Rule] Sin, 
            cos \[Rule] Cos, tan \[Rule] Tan, arcsin \[Rule] ArcSin, 
            arccos \[Rule] ArcCos, arctan \[Rule] ArcTan, 
            gamma \[Rule] Gamma, loggamma \[Rule] LogGamma, 
            polygamma \[Rule] 
              PolyGamma};\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Some\ very\ basic\ simplification\ rules\ *) \[IndentingNewLine]
    \(SimpleSimplifications = {\[IndentingNewLine]power[z_, 0] \[Rule] 
            1, \[IndentingNewLine]power[z_, 1] \[Rule] 
            z, \[IndentingNewLine]power[E, log[z_]] \[Rule] 
            z, \[IndentingNewLine]log[power[E, z_]] \[Rule] 
            z, \[IndentingNewLine]power[power[z_, e_], \(-1\)] \[Rule] 
            power[z, \(-e\)], \[IndentingNewLine]power[\(-power[z_, 
                  e_]\), \(-1\)] \[Rule] \(-power[
                z, \(-e\)]\)\[IndentingNewLine]};\)\)\)\)], "Input",
  CellLabel->"In[205]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      Display\ internal\ functions\ like\ public\ functions\ *) \)\(\
\[IndentingNewLine]\)\(\[IndentingNewLine]\)\(\(log /: 
        MakeBoxes[log[x_], form_] := 
        MakeBoxes[Log[x], form];\)\[IndentingNewLine]
    \(power /: MakeBoxes[power[x_, y_], form_] := 
        MakeBoxes[Power[x, y], form];\)\[IndentingNewLine]\[IndentingNewLine]
    \(Unprotect[Times];\)\[IndentingNewLine]
    \(Times /: MakeBoxes[HoldPattern[Times[t___]], form_] := 
        Module[{h, i}, \[IndentingNewLine]h = 
              HoldComplete[
                MakeBoxes[Times[t], 
                  form]]; \[IndentingNewLine]Do[\[IndentingNewLine]\(If[
                  h[\([1, 1, i, 0]\)] === 
                    power, \[IndentingNewLine]\(h[\([1, 1, i, 0]\)] = 
                      Power;\)\[IndentingNewLine]];\) (*end\ if\
*) \[IndentingNewLine] (*do\ for*) , \[IndentingNewLine]{i, 1, 
                Length[{t}]}\[IndentingNewLine]]; \
\[IndentingNewLine]ReleaseHold[h]\[IndentingNewLine]] /; 
          MemberQ[{t}, power[___]];\)\[IndentingNewLine]
    \(Protect[Times];\)\[IndentingNewLine]\[IndentingNewLine]
    \(sin /: MakeBoxes[sin[x_], form_] := 
        MakeBoxes[Sin[x], form];\)\[IndentingNewLine]
    \(cos /: MakeBoxes[cos[x_], form_] := 
        MakeBoxes[Cos[x], form];\)\[IndentingNewLine]
    \(tan /: MakeBoxes[tan[x_], form_] := 
        MakeBoxes[Tan[x], form];\)\[IndentingNewLine]
    \(arcsin /: MakeBoxes[arcsin[x_], form_] := 
        MakeBoxes[ArcSin[x], form];\)\[IndentingNewLine]
    \(arccos /: MakeBoxes[arccos[x_], form_] := 
        MakeBoxes[ArcCos[x], form];\)\[IndentingNewLine]
    \(arctan /: MakeBoxes[arctan[x_], form_] := 
        MakeBoxes[ArcTan[x], form];\)\[IndentingNewLine]
    \(gamma /: MakeBoxes[gamma[x_], form_] := 
        MakeBoxes[Gamma[x], form];\)\[IndentingNewLine]
    \(loggamma /: MakeBoxes[loggamma[x_], form_] := 
        MakeBoxes[LogGamma[x], form];\)\[IndentingNewLine]
    \(polygamma /: MakeBoxes[polygamma[n_, x_], form_] := 
        MakeBoxes[PolyGamma[n, x], form];\)\)\)\)], "Input",
  CellLabel->"In[86]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      General\ Tools\ *) \)\(\[IndentingNewLine]\)\(\[IndentingNewLine]\)\( \
(*\ Extract\ context\ part\ of\ module\ variables\ *) \)\(\[IndentingNewLine]\
\)\(\(GetSymbolModuleContext[a_Symbol] := 
        Module[{n, p}, \[IndentingNewLine]n = 
            SymbolName[Unevaluated[a]]; \[IndentingNewLine]p = 
            StringPosition[n, "\<$\>"]; \[IndentingNewLine]If[
            p \[Equal] {}, "\<\>", 
            StringDrop[n, 
              p[\([1, 1]\)] - 1]]\[IndentingNewLine]];\)\[IndentingNewLine]
    \(SetAttributes[GetSymbolModuleContext, 
        HoldFirst];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Extract\ variable\ part\ of\ module\ variables\ *) \[IndentingNewLine]
    \(GetSymbolModuleName[a_Symbol] := 
        Module[{n, p}, \[IndentingNewLine]n = 
            StringJoin[Context[Unevaluated[a]], 
              SymbolName[Unevaluated[a]]]; \[IndentingNewLine]p = 
            StringPosition[n, "\<$\>"]; \[IndentingNewLine]If[p \[Equal] {}, 
            n, StringTake[n, 
              p[\([1, 1]\)] - 1]]\[IndentingNewLine]];\)\[IndentingNewLine]
    \(SetAttributes[GetSymbolModuleName, 
        HoldFirst];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Translate\ module\ symbol\ name\ to\ global\ symbol\ name\ \
*) \[IndentingNewLine]
    \(Public[a_Symbol] := 
        Symbol[GetSymbolModuleName[Unevaluated[a]]];\)\[IndentingNewLine]
    \(SetAttributes[Public, 
        HoldFirst];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Change\ a\ symbol' 
        s\ module\ context\ to\ a\ different\ one\ *) \[IndentingNewLine]
    SetModuleContext[a_Symbol, context_] := 
      Symbol[StringJoin[GetSymbolModuleName[Unevaluated[a]], 
          context]]\[IndentingNewLine]
    \(SetAttributes[SetModuleContext, 
        HoldFirst];\)\[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine]\
 (*\ Variable\ options\ handler\ *) \[IndentingNewLine] (*\ 
      Handles\ optional\ parameters\ of\ name \[Rule] 
        value\ form\ *) \[IndentingNewLine] (*\ \(Example\)\(:\)\ \
*) \[IndentingNewLine] (*\ \(MyFunction::"\<UnknownOption\>"\)\(=\)\ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ *) \[IndentingNewLine] (*\ \
\ \ \("\<MyFunction does not support this option: `1`\>";\)\ \ \ \ \ \ \ \ \ \
*) \[IndentingNewLine] (*\ \(Options[MyFunction] = {Foo \[Rule] True, 
            Bar \[Rule] 
              False};\)\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
*) \[IndentingNewLine] (*\ 
      MyFunction[Baz_, 
          Options___]\  := \ \(\(Module\)\([\)\(\({MyFoo, 
            MyBar}\)\(,\)\)\(\ \ \ \ \ \ \)\)*) \[IndentingNewLine] (*\ \ \ \
\(ReadOptions[MyFunction, {Foo \[Rule] MyFoo, Bar \[Rule] MyBar}, 
          Options];\)\ \ *) \[IndentingNewLine] (*\ \ \ \(Print["\<Foo=\>", 
          MyFoo, "\< Bar=\>", 
          MyBar];\)\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
*) \[IndentingNewLine] (*\(\(\(\ \)\(]\)\);\)\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ *) \[IndentingNewLine]\[IndentingNewLine]
    \(Clear[ReadOptions];\)\[IndentingNewLine]
    \(ReadOptions[Caller_, OptionList_, OptionSeq___] := 
        Module[{FullOptions, pos}, \[IndentingNewLine] (*\ List\ of\ Option, 
            Destination, Default\ *) \[IndentingNewLine]FullOptions = 
            Map[{#[\([1]\)], Null, #[\([2]\)]} &, 
              Options[Caller]]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
            Add\ Destinations\ \
*) \[IndentingNewLine]Map[\[IndentingNewLine]\((FullOptions[\([\(Position[
                          FullOptions, {#[\([1]\)], _, _}]\)[\([1, 1]\)], 
                      2]\)] = #[\([2]\)])\) &\[IndentingNewLine], 
            OptionList\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine] (*\ Process\ sequence\ of\ options, \ 
            replace\ defaults\ if\ found\ *) \[IndentingNewLine]Map[\
\[IndentingNewLine]\((\[IndentingNewLine]\(If[\(! MatchQ[#, _ \[Rule] _]\), \
\[IndentingNewLine]\(Message[
                        Caller::"\<UnknownOption\>", #];\)\[IndentingNewLine] \
(*else*) , \[IndentingNewLine]pos = 
                      Position[
                        FullOptions, {#[\([1]\)], _, _}]; \
\[IndentingNewLine]If[
                      Length[pos] \[Equal] 0, \[IndentingNewLine]Message[
                        Caller::"\<UnknownOption\>", #]\[IndentingNewLine] \
(*else*) , \[IndentingNewLine]FullOptions[\([pos[\([1, 1]\)], 
                            3]\)] = #[\([2]\)]\[IndentingNewLine]];\
\[IndentingNewLine]];\)\[IndentingNewLine])\) &, \
\[IndentingNewLine]{OptionSeq}\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine] (*\ 
            Assign\ results\ \
*) \[IndentingNewLine]Map[\((Evaluate[#[\([2]\)]] = #[\([3]\)])\) &, 
            Select[FullOptions, \(! MatchQ[#, {_, 
                      Null, _}]\) &]];\[IndentingNewLine]];\)\
\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Remove\ some\ options\ from\ a\ sequence\ of\ options\ \
*) \[IndentingNewLine]
    \(Clear[DropOptions];\)\[IndentingNewLine]
    \(DropOptions[DropOpts_, Options___] := 
        Module[{opt, 
            i}, \[IndentingNewLine]opt = {Options}; \[IndentingNewLine]Do[\
\[IndentingNewLine]\(opt = 
                Select[opt, \((\(! MatchQ[#, 
                          DropOpts[\([i]\)] \[Rule] _]\))\) &];\)\
\[IndentingNewLine] (*do\ on*) , \[IndentingNewLine]{i, 1, 
              Length[DropOpts]}\[IndentingNewLine]]; \[IndentingNewLine]Apply[
            Sequence, 
            opt]\[IndentingNewLine]];\)\[IndentingNewLine]\[IndentingNewLine] \
(*\ Debug\ Print\ function\ *) \[IndentingNewLine] (*\ 
      IndentPrint\ allows\ to\ indent\ print\ output\ *) \[IndentingNewLine]\
\[IndentingNewLine]
    \(IndentPrint[Spacing_, Text__] := 
        Print[DisplayForm[
            AdjustmentBox[ToBoxes[SequenceForm[Text]], 
              BoxMargins -> \n\ \ \ \ \ {{Spacing, 0}, \ {0, \ 
                    0}}]]];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Interface\ to\ set\ own\ debug\ printing\ routines\ \
*) \[IndentingNewLine]\[IndentingNewLine]
    \($DebugPrint = IndentPrint;\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Interface\ to\ modify\ debug\ and\ message\ output\ \
*) \[IndentingNewLine]\[IndentingNewLine]
    \($OutputProcessor = Identity;\)\)\)\)], "Input",
  CellLabel->"In[101]:=",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Zero Test", "Subsection",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      Global\ Zero\ Test\ code\ *) \)\(\[IndentingNewLine]\)\(\
\[IndentingNewLine]\)\(\(MrvTestZero[term_, x_] := 
        Simplify[
          FromInternal[term, x] == 
            0];\)\(\[IndentingNewLine]\)\(\[IndentingNewLine]\)
    \($MrvTestZero = MrvTestZero;\)\(\[IndentingNewLine]\)\( (*\ 
      test\ if\ constant\ is\ 0\ *) \)\(\[IndentingNewLine]\)\(\
\[IndentingNewLine]\)
    \(MrvTestZeroInterval[term_, x_ \[Rule] \[Infinity]] := 
        Simplify[
          FromInternal[term, x] == 
            0];\)\(\[IndentingNewLine]\)\(\[IndentingNewLine]\)
    \($MrvTestZeroInterval = 
        MrvTestZeroInterval;\)\(\[IndentingNewLine]\)\( (*\ 
      test\ if\ function\ is\ identical\ 0\ in\ interval\ \((x0, \[Infinity])\
\)\ for\ some\ \(\(x0\)\(.\)\)\ *) \)\(\[IndentingNewLine]\)
    \( (*\ Sorry, \ 
      no\ idea\ how\ to\ check\ this\ \(\(properly\)\(.\)\)\ *) \)\)\)\)], \
"Input",
  CellLabel->"In[116]:=",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["MrvSet", "Subsection",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      MrvSet\ is\ a\ helper\ function\ to\ calculate\ the\ set\ of\ most\ \
varying\ sub\ \(\(expressions\)\(.\)\)\ *) \)\(\[IndentingNewLine]\)\( (*\ 
      Call\ MrvSet[f, 
          x]\ with\ function\ f\ in\ internal\ representation\ and\ variable\ \
\(\(x\)\(.\)\)\ *) \)\(\[IndentingNewLine]\)\(\[IndentingNewLine]\)\( (*\ 
      Result\ is\ a\ list\ of\ MrvF[f, lc, 
          lim]\ \(\(objects\)\(.\)\)\ *) \)\(\[IndentingNewLine]\)\( (*\ 
      f\ represents\ one\ of\ the\ most\ varying\ sub\ expressions . \ 
          f\ is\ either\ x, \ 
      or\ of\ e^_\ \(\(form\)\(.\)\)\ *) \)\(\[IndentingNewLine]\)\( (*\ 
      lc\ is\ \(\(LeafCount[f]\)\(.\)\)\ *) \)\(\[IndentingNewLine]\)\( (*\ 
      lim\ is\ the\ limit\ behavior\ of\ f\ for\ x \[Rule] \[Infinity] . \ 
              This\ can\ only\ be\  + \[Infinity]\ or\ 0. \ *) \)\(\
\[IndentingNewLine]\)\( (*\ 
      The\ resulting\ list\ is\ always\ sorted\ by\ lc, \ 
      lower\ \((simpler)\)\ results\ \(\(first\)\(.\)\)\ *) \)\)\)], "Input",
  CellLabel->"In[121]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      Compare\ two\ terms\ regarding\ varying\ class . \ 
          Optimized\ for\ results\ of\ MrvSet\ *) \)\(\[IndentingNewLine]\)\(\
\(MrvCompareMrvSet[exp1_, exp2_, x_] := 
        Module[{l, e1, e2}, \[IndentingNewLine]If[exp1 === exp2, 
            Return[0]]; \[IndentingNewLine] (*\ 
            This\ esp . \ catches\ exp1 = x, \ 
            exp2 = x\ *) \[IndentingNewLine]\[IndentingNewLine]If[
            exp1 === 
              x\ , \[IndentingNewLine] (*then*) \[IndentingNewLine]e1 = 
              log[x], \[IndentingNewLine] (*else*) \[IndentingNewLine]If[\(! \
MatchQ[exp1, power[E, _]]\), 
              Message[MrvLimit::"\<Assert\>", "\<Should be exp:\>", 
                exp1]]; \[IndentingNewLine]e1 = 
              exp1[\([2]\)];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]If[
            exp2 === 
              x\ , \[IndentingNewLine] (*then*) \[IndentingNewLine]e2 = 
              log[x], \[IndentingNewLine] (*else*) \[IndentingNewLine]If[\(! \
MatchQ[exp2, power[E, _]]\), 
              Message[MrvLimit::"\<Assert\>", "\<Should be exp:\>", 
                exp2]]; \[IndentingNewLine]e2 = 
              exp2[\([2]\)];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine] (*calculate\ limit\ of\ Log[exp1]/
                Log[exp2]\ recursively*) \[IndentingNewLine]l = 
            MrvLimitInf[e1*power[e2, \(-1\)], x]; \[IndentingNewLine]If[
            l === Null, \[IndentingNewLine]Message[MrvLimit::"\<LimitFail\>", 
              MrvCompareMrvSet, e1/e2]; \[IndentingNewLine]Return[
              Null];\[IndentingNewLine]]; \[IndentingNewLine]If[$MrvTestZero[
              l, x], Return[1]]; \[IndentingNewLine]If[
            Abs[l] \[Equal] \[Infinity], 
            Return[\(-1\)]]; \[IndentingNewLine]Return[
            0];\[IndentingNewLine]];\)\[IndentingNewLine]\[IndentingNewLine] \
(*Calculate\ union\ of\ two\ MrvSet\ sets . \ 
          Drop\ all\ elements\ that\ are\ not\ of\ max\ varying\ \
\(\(class\)\(.\)\)\ *) \[IndentingNewLine]\[IndentingNewLine]
    \(Clear[MrvSetMax];\)\[IndentingNewLine]
    \(MrvSetMax[Set1_, Set2_, x_] := 
        Module[{l, e1, e2}, \[IndentingNewLine]If[
            Head[Set1] =!= List || 
              Head[Set2] =!= List, \[IndentingNewLine]\(Return[
                Null];\)\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]If[Set1 === {}, Return[Set2]]; \[IndentingNewLine]If[
            Set2 === {}, 
            Return[Set1]]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
            Now\ both\ sets\ have\ at\ least\ one\ member\ \
*) \[IndentingNewLine]\[IndentingNewLine]Switch[
            MrvCompareMrvSet[Set1[\([1, 1]\)], Set2[\([1, 1]\)], 
              x], \[IndentingNewLine]1, Set2, \[IndentingNewLine]\(-1\), 
            Set1, \[IndentingNewLine]0, 
            Sort[Union[Set1, 
                Set2], \((#1[\([2]\)] < #2[\([2]\)])\) &], \
\[IndentingNewLine]Null, 
            Null\  (*\ 
              error\ indicator\ \
*) \[IndentingNewLine]]\[IndentingNewLine]];\)\[IndentingNewLine]\
\[IndentingNewLine]\[IndentingNewLine] (*\ \[IndentingNewLine]MrvSetRules : \
\[IndentingNewLine]Apply\ simple\ rules\ to\ Mrv\ Set\ while\ preserving\ \
sort\ structure\ etc . \ 
            Only\ apply\ rule\ sets\ that\ dont\ change\ the\ limiting\ \
\(behavior!\)\[IndentingNewLine]*) \[IndentingNewLine]\[IndentingNewLine]
    \(MrvSetRules[MrvF[f_, lc_, lim_], Rules_List] := 
        Module[{f2}, \[IndentingNewLine] (*\ 
            Apply\ rule\ and\ re - 
              calculate\ leaf\ count\ *) \[IndentingNewLine]f2 = 
            f /. Rules; \[IndentingNewLine]MrvF[f2, LeafCount[f2], 
            lim]\[IndentingNewLine]];\)\[IndentingNewLine]\[IndentingNewLine]
    \(MrvSetRules[S_List, Rules_List] := \[IndentingNewLine] (*\ 
          Apply\ rules\ and\ re - 
            sort\ *) \[IndentingNewLine]\[IndentingNewLine]Sort[\
\[IndentingNewLine]Map[MrvSetRules[#, Rules] &, 
            S], \[IndentingNewLine]\((#1[\([2]\)] < #2[\([2]\)])\) &\
\[IndentingNewLine]];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Calculate\ set\ of\ most\ rapidly\ varying\ sub\ expression, \ 
      main\ code\ *) \[IndentingNewLine]\[IndentingNewLine]
    \(Clear[MrvSet];\)\[IndentingNewLine]
    \(MrvSet[f_, x_] := {} /; FreeQ[f, x];\)\[IndentingNewLine]
    \(MrvSet[x_, x_] := {MrvF[x, 1, \[Infinity]]};\)\[IndentingNewLine]
    \(MrvSet[g_*h_, x_] := 
        MrvSetMax[MrvSet[g, x], MrvSet[h, x], x];\)\[IndentingNewLine]
    \(MrvSet[g_ + h_, x_] := 
        MrvSetMax[MrvSet[g, x], MrvSet[h, x], x];\)\[IndentingNewLine]
    \(MrvSet[power[g_, c_], x_] := 
        MrvSet[g, x] /; FreeQ[c, x];\)\[IndentingNewLine]
    \(MrvSet[log[g_], x_] := MrvSet[g, x];\)\[IndentingNewLine]
    \(MrvSet[power[E, g_], x_] := 
        Module[{l, m}, \[IndentingNewLine]l = 
            MrvLimitInf[g, x]; \[IndentingNewLine]If[
            l \[Equal] Null, \[IndentingNewLine]Message[
              MrvLimit::"\<LimitFail\>", "\<MrvSet\>", 
              g]; \[IndentingNewLine]Return[Null];  (*\ 
              error\ indicator\ *) \[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]If[
            FreeQ[l, DirectedInfinity], \[IndentingNewLine] (*\ 
              Seems\ const . \ 
                  Pass\ through\ Mrv\ \(\(set\)\(.\)\)\ \
*) \[IndentingNewLine]\(Return[
                MrvSet[g, 
                  x]];\)\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]If[
            l === \[Infinity], \[IndentingNewLine] (*\ 
              tends\ to\ e^\[Infinity] . \ 
                  May\ contribute\ to\ Mrv\ set\ *) \[IndentingNewLine]\
\[IndentingNewLine]m = 
              MrvF[\[IndentingNewLine]power[E, 
                  g], \[IndentingNewLine]LeafCount[
                  power[E, 
                    g]], \[IndentingNewLine]\[Infinity]\[IndentingNewLine]]; \
\[IndentingNewLine]Return[
              MrvSetMax[{m}, MrvSet[g, x], x]];\[IndentingNewLine]]; \  (*\ 
            end\ if\ l === \[Infinity]\ *) \[IndentingNewLine]\
\[IndentingNewLine]\[IndentingNewLine]If[
            l === \(-\[Infinity]\), \[IndentingNewLine] (*\ 
              tends\ to\ e^\(-\[Infinity] . \ 
                      May\)\ contribute\ to\ Mrv\ set\ \
*) \[IndentingNewLine]m = 
              MrvF[\[IndentingNewLine]power[E, 
                  g], \[IndentingNewLine]LeafCount[
                  power[E, 
                    g]], \[IndentingNewLine]0\[IndentingNewLine]]; \
\[IndentingNewLine]Return[
              MrvSetMax[{m}, MrvSet[g, x], x]];\[IndentingNewLine]]; \  (*\ 
            end\ if\ l === \(-\[Infinity]\)\ *) \[IndentingNewLine]\
\[IndentingNewLine]\[IndentingNewLine]Message[MrvLimit::"\<UnknownSign\>", 
            l]; \[IndentingNewLine]Return[Null];\[IndentingNewLine]];\)\  (*\ 
      end\ MrvSet[power[E, g_], 
          x_]\ *) \[IndentingNewLine]\[IndentingNewLine] (*\ 
      Functions\ with\ no\ essential\ singularities\ may\ pass\ \(\(through\)\
\(:\)\)\ *) \[IndentingNewLine] (*\ 
      Unsupported\ cases\ need\ to\ be\ filtered\ out, \ 
      of\ \(\(course\)\(.\)\)\ *) \[IndentingNewLine]
    \(MrvSet[sin[f_], x_] := MrvSet[f, x];\)\[IndentingNewLine]
    \(MrvSet[cos[f_], x_] := MrvSet[f, x];\)\[IndentingNewLine]
    \(MrvSet[tan[f_], x_] := MrvSet[f, x];\)\[IndentingNewLine]
    \(MrvSet[arcsin[f_], x_] := MrvSet[f, x];\)\[IndentingNewLine]
    \(MrvSet[arccos[f_], x_] := MrvSet[f, x];\)\[IndentingNewLine]
    \(MrvSet[arctan[f_], x_] := MrvSet[f, x];\)\[IndentingNewLine]
    \(MrvSet[gamma[f_], x_] := MrvSet[f, x];\)\[IndentingNewLine]
    \(MrvSet[loggamma[f_], x_] := MrvSet[f, x];\)\[IndentingNewLine]
    \(MrvSet[polygamma[n_, f_], x_] := 
        MrvSet[f, x];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Catch\ all\ remaining\ *) \[IndentingNewLine]
    \(MrvSet[f_, 
          x_] := \((\[IndentingNewLine]Message[MrvLimit::"\<MrvSetFailed\>", 
            f]; \[IndentingNewLine]Null\[IndentingNewLine])\);\)\)\)\)], \
"Input",
  CellLabel->"In[122]:=",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Complexity", "Subsection",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      Calculate\ the\ complexity\ of\ a\ term\ *) \)\(\[IndentingNewLine]\)\(\
\[IndentingNewLine]\)\(\(Clear[MrvComplexitySet];\)\[IndentingNewLine]
    \(MrvComplexitySet[f_, x_] := {} /; FreeQ[f, x];\)\[IndentingNewLine]
    \(MrvComplexitySet[x_, x_] := {x};\)\[IndentingNewLine]
    \(MrvComplexitySet[g_*h_, x_] := 
        Union[MrvComplexitySet[g, x], 
          MrvComplexitySet[h, x]];\)\[IndentingNewLine]
    \(MrvComplexitySet[g_ + h_, x_] := 
        Union[MrvComplexitySet[g, x], 
          MrvComplexitySet[h, x]];\)\[IndentingNewLine]
    \(MrvComplexitySet[power[g_, c_], x_] := 
        MrvComplexitySet[g, x] /; FreeQ[c, x];\)\[IndentingNewLine]
    \(MrvComplexitySet[log[g_], x_] := 
        Append[MrvComplexitySet[g, x], log[g]];\)\[IndentingNewLine]
    \(MrvComplexitySet[power[E, g_], x_] := 
        Append[MrvComplexitySet[g, x], 
          power[E, g]];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      Functions\ with\ no\ essential\ singularities\ may\ pass\ \(\(through\)\
\(:\)\)\ *) \[IndentingNewLine]
    \(MrvComplexitySet[sin[f_], x_] := 
        MrvComplexitySet[f, x];\)\[IndentingNewLine]
    \(MrvComplexitySet[cos[f_], x_] := 
        MrvComplexitySet[f, x];\)\[IndentingNewLine]
    \(MrvComplexitySet[tan[f_], x_] := 
        MrvComplexitySet[f, x];\)\[IndentingNewLine]
    \(MrvComplexitySet[arcsin[f_], x_] := 
        MrvComplexitySet[f, x];\)\[IndentingNewLine]
    \(MrvComplexitySet[arccos[f_], x_] := 
        MrvComplexitySet[f, x];\)\[IndentingNewLine]
    \(MrvComplexitySet[arctan[f_], x_] := 
        MrvComplexitySet[f, x];\)\[IndentingNewLine]
    \(MrvComplexitySet[gamma[f_], x_] := 
        MrvComplexitySet[f, x];\)\[IndentingNewLine]
    \(MrvComplexitySet[loggamma[f_], x_] := 
        MrvComplexitySet[f, x];\)\[IndentingNewLine]
    \(MrvComplexitySet[polygamma[n_, f_], x_] := 
        MrvComplexitySet[f, x];\)\[IndentingNewLine]\[IndentingNewLine] (*\ 
      catch\ all\ remaining\ *) \[IndentingNewLine]
    \(MrvComplexitySet[f_, 
          x_] := \((\[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedFunction\>", 
            f]; \[IndentingNewLine]HoldForm[
            MrvComplexitySet[f, 
              x]]\[IndentingNewLine])\);\)\[IndentingNewLine]\
\[IndentingNewLine]
    \(MrvComplexity[f_, x_] := 
        Module[{c}, \[IndentingNewLine]c = 
            MrvComplexitySet[f, 
              x]; \[IndentingNewLine]If[\(! FreeQ[c, MrvComplexitySet]\), 
            Return[\[Infinity]]]; \[IndentingNewLine]Return[
            Length[c]];\[IndentingNewLine]];\)\)\)\)], "Input",
  CellLabel->"In[146]:=",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["MrvSeriesHead", "Subsection",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ Calculate\ head\ of\ power\ series\ of\ f, \ 
      dominant\ variable\ \[Omega], \ 
      secondary\ variable\ \(\(x\)\(.\)\)\ *) \)\(\[IndentingNewLine]\)\( (*\ 
      log\[Omega]\ is\ a\ replacement\ term\ for\ Log[\[Omega]]\ and\ can\ be\
\ substituted\ at\ any\ time\ *) \)\(\[IndentingNewLine]\)\(\
\[IndentingNewLine]\)\( (*\ 
      Returns\ {u, ex}\ if\ leading\ term\ is\ u*\[Omega]^
          ex\ *) \)\(\[IndentingNewLine]\)\( (*\ 
      May\ return\ u = 
        0\ only\ if\ f\ is\ identical\ 0. \ *) \)\(\[IndentingNewLine]\)\(\
\[IndentingNewLine]\)\(\[IndentingNewLine]\)\( (*\ 
      SeriesHead\ *) \)\(\[IndentingNewLine]\)\( (*\ 
      Tries\ hard\ to\ interprete\ the\ output\ of\ the\ Series[]\ \
\(\(command\)\(.\)\)\ *) \)\(\[IndentingNewLine]\)\( (*\ 
      Returns\ {c, e}\ for\ leading\ term\ c*
        x^e\ *) \)\(\[IndentingNewLine]\)\(\[IndentingNewLine]\)\(\(Clear[
        SeriesHead];\)\[IndentingNewLine]
    \(SeriesHead[HoldPattern[SeriesData[x_, 0, c_List, nmin_, nmax_, den_]], 
          x_] := \[IndentingNewLine]If[$MrvTestZero[c[\([1]\)], 
              x], \[IndentingNewLine]SeriesHead[
              SeriesData[x, 0, Drop[c, 1], nmin + 1, nmax, 
                den]]\[IndentingNewLine] (*else*) , \
\[IndentingNewLine]{c[\([1]\)], 
              nmin/den}\[IndentingNewLine] (*undetermined*) , \
\[IndentingNewLine]{c[\([1]\)], nmin/den}\[IndentingNewLine]] /; 
          Length[c] > 0;\)\[IndentingNewLine]\[IndentingNewLine]
    \(SeriesHead[f_, x_] := {f, 0} /; FreeQ[f, x];\)\[IndentingNewLine]
    \(\(SeriesHead[c_. *x_^e_. , x_] := {c, e} /; FreeQ[c, x];\) && 
        FreeQ[e, x];\)\[IndentingNewLine]
    \(SeriesHead[f_*g_, x_] := 
        Module[{sf, sg}, \[IndentingNewLine]sf = 
            SeriesHead[f, x]; \[IndentingNewLine]sg = 
            SeriesHead[g, 
              x]; \[IndentingNewLine]If[\(! FreeQ[sf, 
                  SeriesHead]\) || \(! FreeQ[sg, 
                  SeriesHead]\), \[IndentingNewLine]Return[
              sf*sg]\[IndentingNewLine]]; \[IndentingNewLine]Return[{sf[\([1]\
\)]*sg[\([1]\)], 
              sf[\([2]\)] + 
                sg[\([2]\)]}];\[IndentingNewLine]];\)\[IndentingNewLine]\
\[IndentingNewLine]
    \(SeriesHead[f_ + g_] := 
        Module[{sf, sg}, \[IndentingNewLine]sf = 
            SeriesHead[f, x]; \[IndentingNewLine]sg = 
            SeriesHead[g, 
              x]; \[IndentingNewLine]If[\(! FreeQ[sf, 
                  SeriesHead]\) || \(! FreeQ[sg, 
                  SeriesHead]\), \[IndentingNewLine]Return[
              sf + sg]\[IndentingNewLine]]; \[IndentingNewLine]If[
            sf[\([2]\)] < sg[\([2]\)], Return[sf]]; \[IndentingNewLine]If[
            sf[\([2]\)] > sg[\([2]\)], Return[sg]]; \[IndentingNewLine]If[
            sf[\([2]\)] == sg[\([2]\)], 
            Return[{sf[\([1]\)] + sg[\([1]\)], 
                sf[\([2]\)]}]]; \[IndentingNewLine]Return[
            sf + sg];\[IndentingNewLine]];\)\[IndentingNewLine]\
\[IndentingNewLine]\[IndentingNewLine]
    \(Clear[MrvSeriesHead];\)\[IndentingNewLine]
    \(MrvSeriesHead[f_, x_, \[Omega]_, log\[Omega]_, dprint_] := 
        Module[{ser, serh, i, u, 
            ex}, \[IndentingNewLine]\[IndentingNewLine] (*\ 
            Define\ some\ local\ auto\ simplifications\ \
*) \[IndentingNewLine]Unprotect[Log]; 
          Unprotect[Power]; \[IndentingNewLine]Log[\[Omega]] = 
            FromInternal[log\[Omega], x]; \[IndentingNewLine]Log[
              1/\[Omega]] = \(-Log[\[Omega]]\); \[IndentingNewLine]Power[E, 
              Log[\[Omega]]] = \[Omega]; \[IndentingNewLine]Power[E, 
              Log[1/\[Omega]]] = 1/\[Omega]; \[IndentingNewLine]Protect[Log]; 
          Protect[Power]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
            Call\ Series\ with\ increasing\ order\ to\ find\ terms\ \
*) \[IndentingNewLine] (*\ Ugly, \ 
            but\ not\ to\ \(\(avoid\)\(.\)\)\ *) \[IndentingNewLine]Do[\
\[IndentingNewLine]ser = 
              Series[FromInternal[f, x], {\[Omega], 0, 
                  i}]; \[IndentingNewLine]serh = 
              SeriesHead[
                ser, \[Omega]]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
              check\ for\ series\ term\ that\ run\ out\ of\ precision\ \
*) \[IndentingNewLine]If[
              FreeQ[serh, 
                HoldPattern[
                  SeriesData[_, _, {}, _, _, _]]], \[IndentingNewLine] (*\ 
                No : \ \(\(done\)\(.\)\)\ *) \[IndentingNewLine]\(Break[];\)\
\[IndentingNewLine]];\[IndentingNewLine]\[IndentingNewLine] (*\ 
              Increase\ precision\ and\ continue\ *) \[IndentingNewLine]\
\[IndentingNewLine] (*\ do\ for\ *) \[IndentingNewLine], {i, 1, 
              30}\[IndentingNewLine]]; \  (*\ 
            end\ do\ *) \[IndentingNewLine]\[IndentingNewLine] (*\ 
            Remove\ local\ auto\ simplifications\ \
*) \[IndentingNewLine]Unprotect[Log]; 
          Unprotect[
            Power]; \[IndentingNewLine]Power[E, 
              Log[\[Omega]]] =. ; \[IndentingNewLine]Power[E, 
              Log[1/\[Omega]]] =. ; \[IndentingNewLine]Log[\[Omega]] =. ; \
\[IndentingNewLine]Log[1/\[Omega]] =. ; \[IndentingNewLine]Protect[Log]; 
          Protect[Power]; \
\[IndentingNewLine]\[IndentingNewLine]dprint["\<Series expansion: \>", \
$OutputProcessor[
              ser]]; \[IndentingNewLine]\[IndentingNewLine]If[\(! FreeQ[serh, 
                SeriesHead]\), \[IndentingNewLine] (*\ 
              SeriesHead\ could\ not\ interprete\ the\ result\ \
*) \[IndentingNewLine]If[\(! FreeQ[serh, 
                  HoldPattern[
                    SeriesData[_, _, {}, _, _, _]]]\), \[IndentingNewLine] \
(*\ No\ leading\ terms\ found\ *) \[IndentingNewLine]Message[
                MrvLimit::"\<SeriesNoTerms\>", $OutputProcessor[
                  f]]; \[IndentingNewLine]Return[
                Null];\[IndentingNewLine]]; \[IndentingNewLine]Message[
              MrvLimit::"\<SeriesFail\>", $OutputProcessor[
                f]]; \[IndentingNewLine]Return[
              Null];\[IndentingNewLine]]; \  (*\ 
            end\ if\ *) \[IndentingNewLine]\[IndentingNewLine]{u, ex} = 
            serh; \[IndentingNewLine]\[IndentingNewLine] (*\ 
            Check\ if\ Series\ succeded\ in\ eliminating\ \[Omega]\ \
*) \[IndentingNewLine]If[\(! FreeQ[u, \[Omega]]\), \[IndentingNewLine]Message[
              MrvLimit::"\<SeriesFail\>", $OutputProcessor[
                f]]; \[IndentingNewLine]Return[
              Null];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]u = 
            ToInternal[u, 
              x]; \[IndentingNewLine]If[\(! FreeQ[u, 
                ToInternal]\), \[IndentingNewLine]\(Return[
                Null];\)\[IndentingNewLine]]; \[IndentingNewLine]ex = 
            ToInternal[ex, 
              x]; \[IndentingNewLine]If[\(! FreeQ[ex, 
                ToInternal]\), \[IndentingNewLine]\(Return[
                Null];\)\[IndentingNewLine]]; \[IndentingNewLine]{u, 
            ex}\[IndentingNewLine]];\)\ \  (*\ 
      end\ MrvSeriesHead\ *) \[IndentingNewLine]\[IndentingNewLine]
    \($MrvSeriesHead = MrvSeriesHead;\)\)\)\)], "Input",
  CellLabel->"In[166]:=",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["MrvLimit", "Subsection",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      Helper\ that\ interprets\ leading\ term\ results\ *) \)\(\
\[IndentingNewLine]\)\(\[IndentingNewLine]\)\(\(MrvLimitLeadTermDominant[
          asympt_List] := 
        Module[{i}, \[IndentingNewLine]i = 1; \[IndentingNewLine]While[
            i \[LessEqual] Length[asympt]\  && \ 
              asympt[\([i, 2]\)] \[Equal] 0, \(i++\)]; \[IndentingNewLine]If[
            i > Length[asympt], Return[Null]]; \[IndentingNewLine]Return[
            i];\[IndentingNewLine]];\)\[IndentingNewLine]\[IndentingNewLine]
    \(Clear[MrvLimitOutput];\)\[IndentingNewLine]
    \(MrvLimitOutput[LeadTerm[const_, asympt_List], Output \[Rule] Limit] := 
        Module[{i, ex}, \[IndentingNewLine]i = 
            MrvLimitLeadTermDominant[
              asympt]; \[IndentingNewLine]\[IndentingNewLine]If[i === Null, 
            Return[const]]; \  (*\ All\ exponents\ are\ 0, \ 
            const\ is\ the\ \(\(result\)\(.\)\)\ *) \[IndentingNewLine]ex = 
            asympt[\([i, 2]\)]; \  (*\ 
            Exponent\ of\ leading\ term\ *) \[IndentingNewLine]If[ex > 0, 
            Return[0]];  (*\ 
            tends\ to\ 0\ in\ all\ cases\ *) \[IndentingNewLine]If[
            ex < 0\ , \[IndentingNewLine]\  (*\ 
              limit\ result\ is\  + \(\(/\)\(-\ \[Infinity]\)\)\ \
*) \[IndentingNewLine]s = Sign[const]; \[IndentingNewLine]If[
              s === 0, \[IndentingNewLine]Message[
                MrvLimit::"\<ZeroFunction\>"]; \[IndentingNewLine]Return[
                Null];\[IndentingNewLine]]; \[IndentingNewLine]Return[
              s*\[Infinity]]\[IndentingNewLine]]; \  (*\ 
            end\ if\ ex < 0\ *) \[IndentingNewLine]Message[
            MrvLimit::"\<UnknownSign\>", ex];\[IndentingNewLine]];\)\  (*\ 
      end\ MrvLimitOutput\ *) \[IndentingNewLine]\[IndentingNewLine]
    \(MrvLimitOutput[LeadTerm[const_, asympt_List], Output \[Rule] LeadTerm] := 
        LeadTerm[const, asympt];\)\[IndentingNewLine]\[IndentingNewLine]
    \(MrvLimitOutput[LeadTerm[const_, asympt_List], 
          Output \[Rule] SignedZero] := 
        Module[{l}, \[IndentingNewLine]l = 
            MrvLimitOutput[LeadTerm[const, asympt], 
              Output \[Rule] Limit]; \[IndentingNewLine]If[l === 0, 
            Return[Sign[const]*Zero]]; \[IndentingNewLine]Return[
            l];\[IndentingNewLine]];\)\)\)\)], "Input",
  CellLabel->"In[169]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      Mrv\ limit\ remember\ cache\ \
*) \)\(\[IndentingNewLine]\)\(\(MrvLimitClearCache[] := 
        Module[{}, \[IndentingNewLine]Clear[
            MrvLimitCache]; \[IndentingNewLine]MrvLimitCacheHits = 
            0; \[IndentingNewLine]MrvLimitCacheMisses = 
            0;\[IndentingNewLine]];\)\[IndentingNewLine]
    \(MrvLimitClearCache[];\)\[IndentingNewLine]
    MrvLimitGetCacheStats[] := {CacheHits -> MrvLimitCacheHits, 
        CacheMisses -> MrvLimitCacheMisses}\)\)\)], "Input",
  CellLabel->"In[174]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      Pre - \(Processing : \ 
          Transform\ expression\ into\ processable\ form\ and\ check\ for\ \
limit\ fail\ \(\(conditions\)\(.\)\)\)\ *) \)\(\[IndentingNewLine]\)\(\
\[IndentingNewLine]\)\(\(TestLimitArgQ[f_, x_, test_, 
          Undetermined \[Rule] undet_] := 
        Module[{ff}, \[IndentingNewLine] (*\ 
            test\ if\ limit\ exists\ and\ matches\ a\ test\ criteria\ \
*) \[IndentingNewLine]ff = MrvLimitInf[f, x]; \[IndentingNewLine]If[
            ff === Null, Return[False]]; \[IndentingNewLine]If[
            test[ff], \[IndentingNewLine] \
(*then*) \[IndentingNewLine]\(Return[
                True];\)\[IndentingNewLine],  \
(*else*) \[IndentingNewLine]\(Return[
                False];\)\[IndentingNewLine],  (*undetermined\
*) \[IndentingNewLine]If[undet === True, Return[True]]; \[IndentingNewLine]If[
              undet === False, Return[False]]; \[IndentingNewLine]Message[
              MrvLimit::"\<ConditionCheck\>", 
              test[ff]]; \[IndentingNewLine]Return[
              False];\[IndentingNewLine]];\[IndentingNewLine]];\)\
\[IndentingNewLine]
    \(TestLimitArgQ[f_, x_, test_] := 
        TestLimitArgQ[f, x, test, 
          Undetermined \[Rule] \(-1\)];\)\[IndentingNewLine]\
\[IndentingNewLine]
    \(Clear[MrvLimitPreProcess];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[f_, x_] := f /; FreeQ[f, x];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[x_, x_] := x;\)\[IndentingNewLine]
    \(MrvLimitPreProcess[g_*h_, x_] := 
        MrvLimitPreProcess[g, x]*
          MrvLimitPreProcess[h, x];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[g_ + h_, x_] := 
        MrvLimitPreProcess[g, x] + 
          MrvLimitPreProcess[h, x];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[power[g_, c_], x_] := 
        power[MrvLimitPreProcess[g, x], MrvLimitPreProcess[c, x]] /; 
          FreeQ[c, x];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[power[E, g_], x_] := 
        power[E, MrvLimitPreProcess[g, x]];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[power[f_, g_], x_] := 
        power[E, 
          log[MrvLimitPreProcess[f, x]]*
            MrvLimitPreProcess[g, x]];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[log[f_], x_] := 
        log[MrvLimitPreProcess[f, x]];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[sin[f_], x_] := 
        If[TestLimitArgQ[f, x, 
            FreeQ[#, DirectedInfinity] &], \[IndentingNewLine]sin[
            MrvLimitPreProcess[f, x]], \[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedArgument\>", Sin[x], 
            x \[Rule] \[Infinity], Sin[f]]; \[IndentingNewLine]HoldForm[
            MrvLimitPreProcess[sin[f], 
              x]]\[IndentingNewLine]];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[cos[f_], x_] := 
        If[TestLimitArgQ[f, x, 
            FreeQ[#, DirectedInfinity] &], \[IndentingNewLine]cos[
            MrvLimitPreProcess[f, 
              x]]\[IndentingNewLine] (*else*) , \[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedArgument\>", Cos[x], 
            x \[Rule] \[Infinity], Cos[f]]; \[IndentingNewLine]HoldForm[
            MrvLimitPreProcess[cos[f], 
              x]]\[IndentingNewLine]];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[tan[f_], x_] := 
        If[TestLimitArgQ[f, x, 
            FreeQ[#, DirectedInfinity] &], \[IndentingNewLine]tan[
            MrvLimitPreProcess[f, 
              x]]\[IndentingNewLine] (*else*) , \[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedArgument\>", Tan[x], 
            x \[Rule] \[Infinity], Tan[f]]; \[IndentingNewLine]HoldForm[
            MrvLimitPreProcess[tan[f], 
              x]]\[IndentingNewLine]];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[arcsin[f_], x_] := 
        If[TestLimitArgQ[f, 
            x, \((Abs[#] \[LessEqual] 1)\) &], \[IndentingNewLine]arcsin[
            MrvLimitPreProcess[f, 
              x]]\[IndentingNewLine] (*else*) , \[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedArgument\>", ArcSin[x], 
            x \[Rule] \[Infinity], ArcSin[f]]; \[IndentingNewLine]HoldForm[
            MrvLimitPreProcess[arcsin[f], 
              x]]\[IndentingNewLine]];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[arccos[f_], x_] := 
        If[TestLimitArgQ[f, 
            x, \((Abs[#] \[LessEqual] 1)\) &], \[IndentingNewLine]arccos[
            MrvLimitPreProcess[f, 
              x]]\[IndentingNewLine] (*else*) , \[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedArgument\>", ArcCos[x], 
            x \[Rule] \[Infinity], ArcCos[f]]; \[IndentingNewLine]HoldForm[
            MrvLimitPreProcess[arccos[f], 
              x]]\[IndentingNewLine]];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[arctan[f_], x_] := 
        If[TestLimitArgQ[f, x, \((True)\) &], \[IndentingNewLine]arctan[
            MrvLimitPreProcess[f, 
              x]]\[IndentingNewLine] (*else*) , \[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedArgument\>", ArcTan[x], x, 
            ArcTan[f]]; \[IndentingNewLine]HoldForm[
            MrvLimitPreProcess[arctan[f], 
              x]]\[IndentingNewLine]];\)\[IndentingNewLine]
    \(MrvLimitPreProcess[gamma[f_], x_] := \[IndentingNewLine]If[
          TestLimitArgQ[f, x, 
            FreeQ[#, DirectedInfinity] &], \[IndentingNewLine]gamma[
            MrvLimitPreProcess[f, 
              x]]\[IndentingNewLine] (*else*) , \[IndentingNewLine]If[
            TestLimitArgQ[f, 
              x, \((# === \[Infinity])\) &], \[IndentingNewLine]power[E, 
              loggamma[
                MrvLimitPreProcess[f, 
                  x]]]\[IndentingNewLine] (*else*) , \
\[IndentingNewLine]Message[MrvLimit::"\<UnsupportedArgument\>", Gamma[x], 
              x \[Rule] MrvLimitInf[f, x], 
              Gamma[f]]; \[IndentingNewLine]HoldForm[
              MrvLimitPreProcess[gamma[f], 
                x]]\[IndentingNewLine]]\[IndentingNewLine]];\)\
\[IndentingNewLine]\[IndentingNewLine]
    \(MrvLimitPreProcess[loggamma[f_], x_] := 
        If[TestLimitArgQ[f, 
            x, \((0 \[LessEqual] # \[LessEqual] \[Infinity])\) &], \
\[IndentingNewLine]loggamma[
            MrvLimitPreProcess[f, 
              x]]\[IndentingNewLine] (*else*) , \[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedArgument\>", LogGamma[x], 
            x \[Rule] MrvLimitInf[f, x], 
            LogGamma[f]]; \[IndentingNewLine]HoldForm[
            MrvLimitPreProcess[loggamma[f], 
              x]]\[IndentingNewLine]];\)\[IndentingNewLine]\[IndentingNewLine]\

    \(MrvLimitPreProcess[polygamma[n_, f_], x_] := 
        Module[{}, \[IndentingNewLine]If[\(! IntegerQ[n]\) || 
              n < 0, \[IndentingNewLine]Message[
              MrvLimit::"\<UnsupportedArgument\>", PolyGamma[k, x], 
              k \[Equal] n, PolyGamma[n, f]]; \[IndentingNewLine]Return[
              HoldForm[
                MrvLimitPreProcess[polygamma[n, f], 
                  x]]];\[IndentingNewLine]]; \[IndentingNewLine]If[
            TestLimitArgQ[f, 
              
              x, \((\(-\[Infinity]\) < # \[LessEqual] \[Infinity])\) &], \
\[IndentingNewLine]\(Return[
                polygamma[n, 
                  f]];\)\[IndentingNewLine]]; \[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedArgument\>", PolyGamma[n, x], 
            x \[Rule] MrvLimitInf[f, x], 
            PolyGamma[n, f]]; \[IndentingNewLine]Return[
            HoldForm[
              MrvLimitPreProcess[polygamma[n, f], 
                x]]];\[IndentingNewLine]];\)\[IndentingNewLine]\
\[IndentingNewLine] (*\ catch\ all\ remaining\ *) \[IndentingNewLine]
    \(MrvLimitPreProcess[f_, 
          x_] := \((\[IndentingNewLine]Message[
            MrvLimit::"\<UnsupportedFunction\>", 
            f]; \[IndentingNewLine]HoldForm[
            MrvLimitPreProcess[f, x]]\[IndentingNewLine])\);\)\)\)\)], "Input",\

  CellLabel->"In[177]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      MrvLimitInf : \ 
        Main\ algorithm\ \
*) \)\(\[IndentingNewLine]\)\(\[IndentingNewLine]\)\( (*\ 
      Reset\ Context\ links\ \
*) \)\(\[IndentingNewLine]\)\(\(MrvLimitCurrentContext = 
        Null;\)\[IndentingNewLine]
    \(Clear[MrvLimitRunningTasks];\)\[IndentingNewLine]
    \(MrvLimitDefaultDebugLevel = 
        0;\)\[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine]
    \(Clear[MrvLimitInf];\)\[IndentingNewLine]
    \(Options[MrvLimitInf] = {Output \[Rule] Limit, 
          Debug \[Rule] Automatic};\)\[IndentingNewLine]
    \(MrvLimitInf[ff_, x_, Options___] := 
        Module[\[IndentingNewLine]{\[CapitalOmega], dprint, f, scaleup, w, g, 
            A, s, t, c, a, ex, asympt, i, LastContext, NestLevel, DebugLevel, 
            output}, \[IndentingNewLine]\[IndentingNewLine]ReadOptions[
            MrvLimitInf, {Output \[Rule] output, Debug \[Rule] DebugLevel}, 
            Options]; \[IndentingNewLine]\[IndentingNewLine]If[\(! \
MemberQ[{Limit, LeadTerm, SignedZero}, output]\), \[IndentingNewLine]Message[
              MrvLimit::"\<UnknownOutput\>", \ 
              output]; \[IndentingNewLine]output = 
              Limit;\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine] (*\ 
            Some\ quick\ exits\ for\ really\ simple\ terms\ \
*) \[IndentingNewLine]If[
            FreeQ[ff, x], \[IndentingNewLine]If[output === LeadTerm, 
              Return[LeadTerm[ff, {}]]]; \[IndentingNewLine]Return[
              ff];\[IndentingNewLine]]; \
\[IndentingNewLine]\[IndentingNewLine] (*\ 
            Check\ if\ previous\ MrvLimit\ context\ is\ still\ alive\ \
*) \[IndentingNewLine]If[
            MrvLimitCurrentContext =!= 
              Null, \[IndentingNewLine]\(If[\(! NameQ[
                    StringJoin[GetSymbolModuleName[LastContext], 
                      MrvLimitCurrentContext]]\), \[IndentingNewLine] (*\ 
                  Last\ context\ died\ \(\(unexpected\)\(.\)\)\ \
*) \[IndentingNewLine]Message[
                  MrvLimit::"\<LostContext\>"]; \
\[IndentingNewLine]MrvLimitCurrentContext = Null; \[IndentingNewLine]Clear[
                  MrvLimitRunningTasks];\[IndentingNewLine]];\)\
\[IndentingNewLine]]; \  (*\ 
            end\ if\ *) \[IndentingNewLine]\[IndentingNewLine] (*\ 
            
            Connect\ to\ last\ context\ if\ \(\(present\)\(.\)\)\ \
*) \[IndentingNewLine] (*\ From\ here\ on, \ 
            clean\ exit\ code\ is\ \(\(requred\)\(.\)\)\ \
*) \[IndentingNewLine]LastContext = 
            MrvLimitCurrentContext; \[IndentingNewLine]MrvLimitCurrentContext \
= GetSymbolModuleContext[
              LastContext]; \[IndentingNewLine]\[IndentingNewLine]Catch[\
\[IndentingNewLine] (*\ 
                Catch/Throw\ error\ handling\ from\ here . \ 
                    Throw[]\ uses\ clean\ exit\ \(\(code\)\(.\)\)\ \
*) \[IndentingNewLine]\[IndentingNewLine] (*\ 
                Get\ MrvLimit\ nest\ level\ *) \[IndentingNewLine]NestLevel = 
                If[LastContext === Null, 1, 
                  SetModuleContext[NestLevel, LastContext] + 
                    1]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
                Get\ Debug\ level\ *) \[IndentingNewLine]If[
                DebugLevel === 
                  Automatic, \[IndentingNewLine]\(DebugLevel = 
                    If[LastContext === Null, MrvLimitDefaultDebugLevel, 
                      SetModuleContext[DebugLevel, 
                        LastContext]];\)\[IndentingNewLine]]; \
\[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine] (*\ 
                Prepare\ dprint\ based\ on\ DebugLevel\ \
*) \[IndentingNewLine]If[
                DebugLevel \[GreaterEqual] 
                  NestLevel\[IndentingNewLine] (*then*) , \
\[IndentingNewLine]dprint[s__] := $DebugPrint[NestLevel - 1, 
                    s]; \[IndentingNewLine]dprint["\<Enter Level \>", 
                  NestLevel, "\< Call \>", $OutputProcessor[
                    HoldForm[
                      MrvLimitInf[ff, x, 
                        Options]]]];\[IndentingNewLine] (*else*) , \
\[IndentingNewLine]\(dprint[s__] = 
                    Null;\)\[IndentingNewLine]];  (*end\ if\
*) \[IndentingNewLine]SetAttributes[dprint, 
                HoldAll]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
                Check\ for\ cached\ result\ *) \[IndentingNewLine]asympt = 
                MrvLimitCache[ff]; \[IndentingNewLine]If[
                Head[asympt] =!= 
                  MrvLimitCache, \[IndentingNewLine] (*found\ in\ cache\
*) \[IndentingNewLine]\[IndentingNewLine]\(MrvLimitCacheHits++\); \
\[IndentingNewLine]dprint["\<Taking cached result\>"]; \[IndentingNewLine]\
\[IndentingNewLine]Throw[
                  MrvLimitOutput[asympt, 
                    Output \[Rule] 
                      output]];\[IndentingNewLine]\[IndentingNewLine] (*else\ \
if\ Length[i] > 
                    0*) , \[IndentingNewLine]\(\(MrvLimitCacheMisses++\);\)\
\[IndentingNewLine]]; \  (*\ 
                end\ if\ Length[i] > 
                  0\ *) \[IndentingNewLine]\[IndentingNewLine] (*\ 
                Check\ for\ recursive\ calls\ *) \[IndentingNewLine]If[
                MrvLimitRunningTasks[ff], \[IndentingNewLine]Message[
                  MrvLimit::"\<RecursiveCall\>", 
                  ff]; \[IndentingNewLine]Return[
                  Null];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine] (*\ 
                Mark\ this\ task\ ' 
                  in\ progress'\ for\ recursive\ call\ checking\ \
*) \[IndentingNewLine]MrvLimitRunningTasks[ff] = 
                True; \[IndentingNewLine]\[IndentingNewLine]\
\[IndentingNewLine] \
(*\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\
**\)\(**\)\(**\)******) \[IndentingNewLine] (*\ Ok, \ 
                lets\ start\ with\ the\ real\ work\ *) \[IndentingNewLine] \
(*\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\
**\)\(**\)\(**\)******) \[IndentingNewLine]\[IndentingNewLine]f = 
                ff; \[IndentingNewLine]If[$MrvTestZeroInterval[f, 
                  x \[Rule] \[Infinity]], \[IndentingNewLine] (*\ 
                  Function\ is\ identical\ 0\ around\ \[Infinity] . \ 
                        Dont\ start\ to\ calculate\ crap\ with\ it ... \ \
*) \[IndentingNewLine]\(Throw[
                    0];\)\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]asympt = {}; \[IndentingNewLine] (*\ 
                Result\ is\ f*Apply[Times, asympt] /. pwr \[Rule] Power\ , \ 
                while\ asympt\ is\ a\ dominance\ ordered\ list\ of\ pwr[w, 
                    ex], \ w\ tends\ to\ 0\ for\ x \[Rule] 
                  0. \[IndentingNewLine]*) \[IndentingNewLine]\
\[IndentingNewLine]While[\(! FreeQ[f, x]\), \[IndentingNewLine] (*\ 
                  While\ f\ depends\ on\ \(x : \ 
                      Do\ Mrv\ limit\ calculation\)\ *) \[IndentingNewLine]\
\[IndentingNewLine]dprint["\<Calculating limit of \>", $OutputProcessor[
                    f]]; \[IndentingNewLine]\[IndentingNewLine] (*\ First, \ 
                  do\ pre - 
                    processing\ for\ some\ functions\ *) \[IndentingNewLine]f \
= MrvLimitPreProcess[f, 
                    x]; \[IndentingNewLine]If[\(! FreeQ[f, 
                      MrvLimitPreProcess]\), \[IndentingNewLine]dprint["\<Pre \
Processing failed  :\>", $OutputProcessor[f]]; \[IndentingNewLine]Throw[
                    Null];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine] (*\ 
                  Now\ do\ ' 
                    simple\ simplifications'\ *) \[IndentingNewLine]f = 
                  f /. SimpleSimplifications; \[IndentingNewLine]\
\[IndentingNewLine] (*\ 
                  Calculate\ Mrv\ set\ \
*) \[IndentingNewLine]dprint["\<Calculating Mrv set of \>", $OutputProcessor[
                    f]]; \[IndentingNewLine]\[CapitalOmega] = 
                  MrvSet[f, x]; \[IndentingNewLine]If[\[CapitalOmega] === 
                    Null, Throw[
                    Null]]; \[IndentingNewLine]If[\[CapitalOmega] === {}, \
\[IndentingNewLine]Message[
                    MrvLimit::"\<MrvSetFailed\>", $OutputProcessor[
                      f]]; \[IndentingNewLine]Throw[
                    Null];\[IndentingNewLine]]; \[IndentingNewLine]dprint["\<\
\[CapitalOmega]=\>", $OutputProcessor[\[CapitalOmega]]]; \[IndentingNewLine]\
\[IndentingNewLine] (*\ 
                  Scale\ \(\(up\)\(?\)\)\ *) \[IndentingNewLine]scaleup = 
                  0; \[IndentingNewLine]While[\[CapitalOmega][\([1, 1]\)] === 
                    x, \[IndentingNewLine] (*\ x, \ 
                    if\ present\ in\ \[CapitalOmega], \ 
                    will\ always\ be\ the\ first\ one\ *) \[IndentingNewLine]\
\(scaleup++\); \[IndentingNewLine]f = 
                    f /. {log[x] \[Rule] x, 
                        x \[Rule] 
                          power[E, x]}; \[IndentingNewLine]\[CapitalOmega] = 
                    MrvSetRules[\[CapitalOmega], {log[x] \[Rule] x, 
                        x \[Rule] 
                          power[E, 
                            x]}]; \[IndentingNewLine]\[IndentingNewLine] (*\ \
\[IndentingNewLine]in\ f\ and\ \[CapitalOmega], \ 
                    all\ occurences\ of\ e^
                        x\ got\ replaced\ the\ same\ way . \ also, \ 
                    all\ inner\ occurences\ of\ log[
                        x]\ got\ replaced\ the\ same\ way . \ 
                        Only\ \(problem : \ 
                        The\ x\ in\ \[CapitalOmega]\ may\ have\ been\ present\
\ in\ f\ as\ log[x]\), \ 
                    transforming\ x \[Rule] \(e^
                            x\ in\ \[CapitalOmega]\ and\ log[x] \[Rule] 
                        x\ in\ f . \ 
                            But\ since\ all\ members\ of\ \[CapitalOmega]\ \
now\ have\ comparability\ class\ e^x\), \ 
                    this\ x\ term\ is\ of\ lower\ class . \ 
                        But\ if\ none\ of\ the\ terms\ in\ \[CapitalOmega]\ \
is\ left\ in\ f\ \((all\ occurences\ of\ x\ in\ f\ were\ log[x])\), \ 
                    then\ there\ is\ no\ term\ of\ e^x\ order\ in\ f, \ 
                    and\ we\ have\ to\ search\ again\ for\ the\ next\ lower\ \
\(\(class\)\(.\)\)\[IndentingNewLine]*) \[IndentingNewLine]\[IndentingNewLine]\
\[CapitalOmega] = Select[\[CapitalOmega], \(! FreeQ[
                            f, #[\([1]\)]]\) &]; \[IndentingNewLine] (*\ 
                    eliminate\ non - 
                      occuring\ terms\ *) \[IndentingNewLine]\
\[IndentingNewLine]If[\[CapitalOmega] === {}, \[CapitalOmega] = 
                      MrvSet[f, x]]; \[IndentingNewLine] (*\ 
                    If\ all\ eliminated, \ 
                    scan\ for\ next\ lower\ class\ *) \[IndentingNewLine]\
\[IndentingNewLine]dprint["\<Scale Up: f=\>", $OutputProcessor[
                      f], "\<\n\>", "\<\[CapitalOmega]=\>", $OutputProcessor[\
\[CapitalOmega]]];\[IndentingNewLine]]; \
\[IndentingNewLine]\[IndentingNewLine] (*\ 
                  All\ Mrv' 
                    s\ must\ be\ of\ e^_\ form . \ 
                      Assert\ if\ \(\(not\)\(.\)\)\ *) \[IndentingNewLine]If[\
\(! Apply[And, Map[MatchQ[#[\([1]\)], 
                            power[E, _]] &, \[CapitalOmega]]]\), \
\[IndentingNewLine]Message[
                    MrvLimit::"\<Assert\>", "\<\[CapitalOmega] \
non-exponential\>", \[CapitalOmega]]; \[IndentingNewLine]Throw[
                    Null];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine] (*\ \[Omega]\ is\ the\ first\ one\ \
*) \[IndentingNewLine]w = \[CapitalOmega][\([1, 
                      1]\)]; \[IndentingNewLine]\[IndentingNewLine]dprint["\<\
\[Omega]=\>", $OutputProcessor[
                    w]]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
                  We\ need\ \[Omega] \[Rule] \(0. \ If\ \[Omega] \[Rule] \
\[Infinity]\), \ \(\(substitute\)\(.\)\)\ *) \[IndentingNewLine] (*\ 
                  We\ know, \ \[Omega]\ is\ of\ e^
                      f_\ form\ *) \[IndentingNewLine]\[IndentingNewLine]If[\
\[CapitalOmega][\([1, 3]\)] === \[Infinity], \[IndentingNewLine] (*\ 
                    w \[Rule] \(\[Infinity] . \ 
                            Turn\ around\ sign\ inside\ e^_\ to\ get\ w \
\[Rule] 0\)\ *) \[IndentingNewLine]w[\([2]\)] = \(-w[\([2]\)]\); \
\[IndentingNewLine]dprint["\<Replacing \[Omega]\[Rule]1/\[Omega]: \
\[Omega]=\>", $OutputProcessor[
                      w]];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine] (*\ 
                  replace\ to\ \[Omega]\ in\ f\ step\ by\ step\ \
*) \[IndentingNewLine]Do[\[IndentingNewLine]g = \[CapitalOmega][\([i, 
                        1]\)]; \ \[IndentingNewLine] (*\ 
                    Substitute\ all\ occurences\ of\ this\ subexpr\ \
*) \[IndentingNewLine] (*\ 
                    see\ chapter\ 4.8\ or\ Lemma\ 3.18\ [
                        Gru96]\ for\ details\ \
*) \[IndentingNewLine]dprint["\<Prepare to rewrite g=\>", $OutputProcessor[
                      g], "\< to A*\[Omega]^c\>"]; \[IndentingNewLine]\
\[IndentingNewLine]c = 
                    FromInternal[g[\([2]\)]/w[\([2]\)], 
                      x]; \[IndentingNewLine] (*\ 
                    We' ll\ take\ some\ auto - 
                      optimizations\ for\ this\ *) \[IndentingNewLine]c = 
                    ToInternal[c, 
                      x]; \[IndentingNewLine]If[\(! FreeQ[c, 
                        ToInternal]\), \[IndentingNewLine]\(c = 
                        Null;\)\[IndentingNewLine]]; \[IndentingNewLine] (*\ 
                    Now\ fine\ again\ *) \[IndentingNewLine]\
\[IndentingNewLine]c = MrvLimitInf[c, x]; \[IndentingNewLine]If[
                    c === Null\  || \ 
                      c === 0\  || \ \(! FreeQ[c, 
                          DirectedInfinity]\), \[IndentingNewLine]Message[
                      MrvLimit::"\<LimitFail\>", "\<MrvLimit rewrite\>", 
                      g[\([2]\)]/w[\([2]\)]]; \[IndentingNewLine]Throw[
                      Null]\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]A = 
                    power[E, g[\([2]\)] - c\ w[\([2]\)]] //. 
                      SimpleSimplifications; \[IndentingNewLine]\
\[IndentingNewLine]dprint["\<c=\>", $OutputProcessor[
                      c], "\< A=\>", $OutputProcessor[
                      A]]; \[IndentingNewLine]\[IndentingNewLine]f = 
                    f /. {g \[Rule] A*power[\[Omega], c]}; \  (*\ 
                    and\ \(\(replace\)\(.\)\)\ *) \[IndentingNewLine]\
\[IndentingNewLine]dprint[\[IndentingNewLine]"\<Rewriting \>", \
$OutputProcessor[g], "\<\[Rule]\>", $OutputProcessor[
                      A*power[\[Omega], 
                          c]], \[IndentingNewLine]"\<\n\>", "\<f=\>", \
$OutputProcessor[
                      f]\[IndentingNewLine]];\[IndentingNewLine]\
\[IndentingNewLine] (*do\ for*) \[IndentingNewLine], {i, 
                    Length[\[CapitalOmega]], 
                    1, \(-1\)}\[IndentingNewLine]]; \  (*\ 
                  end\ do\ *) \[IndentingNewLine]\[IndentingNewLine] (*\ 
                  Done\ rewriting\ \
*) \[IndentingNewLine]\[IndentingNewLine]dprint["\<Calculating power series: \
f=\>", $OutputProcessor[f]]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
                  Calculate\ power\ series\ head\ *) \[IndentingNewLine]ser = \
$MrvSeriesHead[f, x, \[Omega], w[\([2]\)], 
                    dprint]; \[IndentingNewLine]\[IndentingNewLine]If[
                  ser === Null, Throw[Null]]; \[IndentingNewLine]{f, ex} = 
                  ser; \[IndentingNewLine]\[IndentingNewLine]While[
                  scaleup > 
                    0, \[IndentingNewLine]\(scaleup--\); \[IndentingNewLine]f \
= f /. {power[E, x] \[Rule] x, x \[Rule] log[x]}; \[IndentingNewLine]w = 
                    w /. {power[E, x] \[Rule] x, 
                        x \[Rule] 
                          log[x]}; \[IndentingNewLine]dprint["\<Scale Down: \
f=\>", $OutputProcessor[f], "\< , \[Omega]=\>", $OutputProcessor[
                      w]];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]dprint["\<Dominant term: \>", $OutputProcessor[
                    f*power[\[Omega], 
                        ex]], "\< , \[Omega]=\>", $OutputProcessor[
                    w]]; \[IndentingNewLine]\[IndentingNewLine]If[$\
MrvTestZeroInterval[f, x \[Rule] \[Infinity]], \[IndentingNewLine] (*\ 
                    This\ explicitly\ means\ that\ f\ has\ been\ identical\ \
zero\ *) \  (*\ \(for\ x \[Rule] \[Infinity]\ when\ entering\ this\ loop . \ 
                          This\ is\ fatal\)\(,\)\ *) \[IndentingNewLine] (*\ 
                    since\ this\ should\ have\ been\ detected\ before\ \
entering\ *) \[IndentingNewLine] (*\ the\ loop, \ 
                    or\ while\ in\ the\ previous\ loop\ \(\(run\)\(.\)\)\ \
*) \[IndentingNewLine] (*\ \(Either\ the\ function\ was\ identical\ 0\ from\ \
the\ beginning\)\(,\)\ *) \[IndentingNewLine] (*\ 
                    or\ one\ of\ the\ series\ heads\ returned\ has\ been\ \
identical\ 0\ *) \[IndentingNewLine] (*\ 
                    without\ detecting\ it . \ Either\ way, \ 
                    we\ failed\ \(\(miserably\)\(.\)\)\ \
*) \[IndentingNewLine]\[IndentingNewLine]Message[
                    MrvLimit::"\<ZeroFunction\>"]; \[IndentingNewLine]Throw[
                    Null];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]AppendTo[asympt, 
                  pwr[w, ex]]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
                  End\ of\ this\ limit\ round . \ 
                      Continue\ on\ lower\ Mrv\ \(\(level\)\(?\)\)\ \
*) \[IndentingNewLine]]; \  (*\ \(While\ !\) 
                  FreeQ[f, 
                    x]\ *) \[IndentingNewLine]\[IndentingNewLine]\
\[IndentingNewLine]\[IndentingNewLine] (*\ 
                Result\ is\ f*Apply[Times, asympt] /. pwr \[Rule] Power\ , \ 
                while\ asympt\ is\ a\ dominance\ ordered\ list\ of\ pwr[w, 
                    ex], \ w\ tends\ to\ 0\ for\ x \[Rule] 
                  0\ \[IndentingNewLine]*) \[IndentingNewLine]\
\[IndentingNewLine]dprint["\<Dominant asymptotic: \>", $OutputProcessor[f], 
                Apply[Sequence, $OutputProcessor[
                    asympt /. 
                      pwr \[Rule] 
                        power]]]; \[IndentingNewLine]\[IndentingNewLine]\
\[IndentingNewLine] (*\ 
                Add\ this\ result\ to\ cache\ \
*) \[IndentingNewLine]MrvLimitCache[ff] = 
                LeadTerm[f, 
                  asympt]; \[IndentingNewLine]\[IndentingNewLine]Throw[
                MrvLimitOutput[LeadTerm[f, asympt], 
                  Output \[Rule] 
                    output]]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
                end\ Catch\ *) \[IndentingNewLine]]\  // \ \((\
\[IndentingNewLine] (*Begin\ pure\ function*) \[IndentingNewLine] (*\ 
                  Exit\ code\ *) \[IndentingNewLine]\[IndentingNewLine] (*\ 
                  Clear\ ' in\ progress'\ state\ *) \[IndentingNewLine]If[
                  MrvLimitRunningTasks[ff], 
                  MrvLimitRunningTasks[
                      ff] =. ]; \
\[IndentingNewLine]\[IndentingNewLine]dprint["\<Leave Level \>", 
                  NestLevel, "\< Call \>", $OutputProcessor[
                    HoldForm[
                      MrvLimitInf[ff, x, 
                          Options] \[Equal] #]]]; \[IndentingNewLine]\
\[IndentingNewLine]MrvLimitCurrentContext = 
                  LastContext; \[IndentingNewLine]\[IndentingNewLine]#\  (*\ 
                  Pass\ through\ result\ *) \[IndentingNewLine])\) &\  (*\ 
            end\ pure\ function\ *) \[IndentingNewLine]];\) (*\ 
      end\ MrvLimitInf\ *) \[IndentingNewLine]\[IndentingNewLine] (*\ 
      force\ cache\ clear\ on\ code\ change\ *) \[IndentingNewLine]
    \(MrvLimitClearCache[];\)\)\)\)], "Input",
  CellLabel->"In[198]:=",
  InitializationCell->True],

Cell[BoxData[
    \(\(\( (*\ 
      MrvLimit\ interface\ code\ *) \)\(\[IndentingNewLine]\)\(\
\[IndentingNewLine]\)\(\(Clear[MrvLimit];\)\[IndentingNewLine]
    \(Options[MrvLimit] = {Output \[Rule] Limit, Debug \[Rule] Automatic, 
          Direction \[Rule] Automatic, 
          ClearCache \[Rule] False};\)\[IndentingNewLine]
    \(MrvLimit[ff_, x_ \[Rule] lim_, Options___] := 
        Module[{Res, ResA, ResB, f, z, direction, lim1, ChildOptions, 
            clearcache}, \[IndentingNewLine]ReadOptions[
            MrvLimit, {Direction \[Rule] direction, 
              ClearCache \[Rule] clearcache}, 
            Options]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
            Remove\ Direction \[Rule] \ \(and\ ClearCache \[Rule] \ 
                options\)\ *) \[IndentingNewLine]ChildOptions = 
            DropOptions[{Direction, ClearCache}, 
              Options]; \[IndentingNewLine]\[IndentingNewLine]If[
            clearcache, \[IndentingNewLine]\(MrvLimitClearCache[];\)\
\[IndentingNewLine]]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
            Delete\ lost\ contexts\ \
*) \[IndentingNewLine]MrvLimitCurrentContext = Null; \[IndentingNewLine]Clear[
            MrvLimitRunningTasks]; \[IndentingNewLine]\[IndentingNewLine]f = 
            ToInternal[ff, 
              x]; \[IndentingNewLine]If[\(! FreeQ[f, 
                ToInternal]\), \[IndentingNewLine]\(Return[
                HoldForm[
                  MrvLimit[ff, x \[Rule] lim, 
                    Options]]];\)\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]lim1 = lim; \[IndentingNewLine]If[
            direction === Automatic, \[IndentingNewLine] (*\ 
              Set\ auto\ direction\ *) \[IndentingNewLine]\(Which[\
\[IndentingNewLine]Head[lim] === Global`SuperPlus, \((direction = \(-1\); 
                  lim1 = lim[\([1]\)])\), \[IndentingNewLine]Head[lim] === 
                  Global`SuperMinus, \((direction = 1; 
                  lim1 = lim[\([1]\)])\), \[IndentingNewLine] (*\ 
                  hell\ knows\ why\ I\ have\ to\ explicitly\ request\ global\ \
context . \ 
                      Wont\ work\ \
\(\(otherwise\)\(.\)\)*) \[IndentingNewLine]lim === \[Infinity], 
                direction = 1, \[IndentingNewLine]lim === \(-\[Infinity]\), 
                direction = \(-1\), \[IndentingNewLine]True, 
                direction = 
                  0\[IndentingNewLine]];\)\[IndentingNewLine]]; \
\[IndentingNewLine]\[IndentingNewLine]If[
            direction \[LessEqual] 0, \[IndentingNewLine] (*\ 
              limit\ from\ above\ *) \[IndentingNewLine]If[
              Abs[lim1] \[Equal] \[Infinity], \[IndentingNewLine]ResA = 
                MrvLimitInf[f /. x \[Rule] \(-x\), x, 
                  ChildOptions]; \[IndentingNewLine]ResA = 
                ResA /. {x \[Rule] \(-x\)};\[IndentingNewLine] (*\ 
                else\ *) , \[IndentingNewLine]ResA = 
                MrvLimitInf[\[IndentingNewLine]\(f /. {x \[Rule] 
                          power[x, \(-1\)] + lim1}\) //. 
                    SimpleSimplifications\[IndentingNewLine], x, 
                  ChildOptions]; \[IndentingNewLine]ResA = \(ResA /. {x \
\[Rule] power[x - lim1, \(-1\)]}\) //. 
                  SimpleSimplifications;\[IndentingNewLine]]; \
\[IndentingNewLine]\[IndentingNewLine]If[\ ResA === Null, 
              Return[HoldForm[
                  MrvLimit[ff, x \[Rule] lim, 
                    Options]]]];\[IndentingNewLine]]; \[IndentingNewLine]If[
            direction \[GreaterEqual] 0, \[IndentingNewLine] (*\ 
              limit\ from\ below\ *) \[IndentingNewLine]If[
              Abs[lim1] \[Equal] \[Infinity], \[IndentingNewLine]\(ResB = 
                  MrvLimitInf[f, x, ChildOptions];\)\[IndentingNewLine] (*\ 
                else\ *) , \[IndentingNewLine]ResB = 
                MrvLimitInf[\[IndentingNewLine]\(f /. {x \[Rule] \(-power[
                                x, \(-1\)]\) + lim1}\) //. 
                    SimpleSimplifications\[IndentingNewLine], x, 
                  ChildOptions]; \[IndentingNewLine]ResB = \(ResB /. {x \
\[Rule] power[lim1 - x, \(-1\)]}\) //. 
                  SimpleSimplifications;\[IndentingNewLine]]; \
\[IndentingNewLine]\[IndentingNewLine]If[\ ResB === Null, 
              Return[HoldForm[
                  MrvLimit[ff, x \[Rule] lim, 
                    Options]]]];\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine] (*\ 
            Final\ result\ processing\ depending\ on\ direction\ \
*) \[IndentingNewLine]\[IndentingNewLine]If[
            direction \[Equal] 0, \[IndentingNewLine] (*\ 
              two\ sided\ limit, \ 
              check\ continuity\ *) \[IndentingNewLine]\(If[
                ResA === ResB\ , \[IndentingNewLine]\(Res = 
                    ResA;\)\[IndentingNewLine], \  (*\ 
                  else\ *) \[IndentingNewLine]\(Res = {ResB, 
                      
                      ResA};\)\[IndentingNewLine]];\)\[IndentingNewLine]]; \
\[IndentingNewLine]If[
            direction < 
              0, \[IndentingNewLine]\(Res = 
                ResA;\)\[IndentingNewLine]]; \[IndentingNewLine]If[
            direction > 
              0, \[IndentingNewLine]\(Res = 
                ResB;\)\[IndentingNewLine]]; \[IndentingNewLine]\
\[IndentingNewLine]If[Head[Res] =!= LeadTerm, 
            Res = FromInternal[Res, 
                x]]; \[IndentingNewLine]\[IndentingNewLine]Res\
\[IndentingNewLine]];\)\  (*end\ MrvLimit*) \[IndentingNewLine]
    \)\)\)], "Input",
  CellLabel->"In[200]:=",
  InitializationCell->True],

Cell[BoxData[{
    \(\(End[];\)\), "\[IndentingNewLine]", 
    \(\(EndPackage[];\)\)}], "Input",
  CellLabel->"In[203]:=",
  InitializationCell->True]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1280}, {0, 887}},
AutoGeneratedPackage->Automatic,
WindowSize->{950, 665},
WindowMargins->{{Automatic, 0}, {Automatic, 6}},
Magnification->1.25,
StyleDefinitions -> "ArticleModern.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 63, 1, 68, "Subsection",
  InitializationCell->True],
Cell[1842, 56, 662, 14, 137, "Input",
  InitializationCell->True],
Cell[2507, 72, 90, 1, 31, "Text",
  InitializationCell->True],
Cell[2600, 75, 120, 3, 53, "Input",
  InitializationCell->True],
Cell[2723, 80, 873, 16, 242, "Input",
  InitializationCell->True],
Cell[3599, 98, 504, 8, 158, "Input",
  InitializationCell->True],
Cell[4106, 108, 260, 5, 95, "Input",
  InitializationCell->True],
Cell[4369, 115, 1165, 17, 347, "Input",
  InitializationCell->True],
Cell[5537, 134, 792, 12, 221, "Input",
  InitializationCell->True],
Cell[6332, 148, 1778, 33, 389, "Input",
  InitializationCell->True],
Cell[8113, 183, 113, 3, 53, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[8263, 191, 56, 1, 68, "Subsection",
  InitializationCell->True],
Cell[8322, 194, 342, 7, 137, "Input",
  InitializationCell->True],
Cell[8667, 203, 531, 10, 200, "Input",
  InitializationCell->True],
Cell[9201, 215, 4917, 95, 1334, "Input",
  InitializationCell->True],
Cell[14121, 312, 2137, 41, 620, "Input",
  InitializationCell->True],
Cell[16261, 355, 6400, 116, 2090, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[22698, 476, 59, 1, 68, "Subsection",
  InitializationCell->True],
Cell[22760, 479, 987, 22, 284, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[23784, 506, 56, 1, 68, "Subsection",
  InitializationCell->True],
Cell[23843, 509, 1096, 19, 221, "Input",
  InitializationCell->True],
Cell[24942, 530, 8058, 148, 2972, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[33037, 683, 60, 1, 68, "Subsection",
  InitializationCell->True],
Cell[33100, 686, 2658, 54, 725, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[35795, 745, 63, 1, 68, "Subsection",
  InitializationCell->True],
Cell[35861, 748, 7184, 131, 2447, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[43082, 884, 58, 1, 68, "Subsection",
  InitializationCell->True],
Cell[43143, 887, 2416, 41, 809, "Input",
  InitializationCell->True],
Cell[45562, 930, 579, 12, 221, "Input",
  InitializationCell->True],
Cell[46144, 944, 7906, 154, 2279, "Input",
  InitializationCell->True],
Cell[54053, 1100, 19959, 347, 6059, "Input",
  InitializationCell->True],
Cell[74015, 1449, 5573, 100, 1901, "Input",
  InitializationCell->True],
Cell[79591, 1551, 150, 4, 74, "Input",
  InitializationCell->True]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

